@@ -1,3 +1,4 @@
+mod action;
 mod app;
 mod args;
 mod cli;
@@ -7,12 +8,19 @@ mod defaults;
 mod drawing;
 mod executor;
 mod failure;
+mod help_line;
+mod help_page;
+mod internal;
 mod job;
+mod job_ref;
+mod job_stack;
+mod keybindings;
 mod line;
 mod line_analysis;
 mod line_type;
 mod list_jobs;
 mod mission;
+mod mission_location;
 mod package_config;
 mod prefs;
 mod report;
@@ -22,8 +30,11 @@ mod state;
 mod stats;
 mod tty;
 mod wrap;
+mod wrapped_command_output;
+mod wrapped_report;
 
 pub use {
+    action::*,
     app::*,
     args::*,
     cli::*,
@@ -33,12 +44,19 @@ pub use {
     drawing::*,
     executor::*,
     failure::*,
+    help_line::*,
+    help_page::*,
+    internal::*,
     job::*,
+    job_ref::*,
+    job_stack::*,
+    keybindings::*,
     line::*,
     line_analysis::*,
     line_type::*,
     list_jobs::*,
     mission::*,
+    mission_location::*,
     package_config::*,
     prefs::*,
     report::*,
@@ -48,6 +66,8 @@ pub use {
     stats::*,
     tty::*,
     wrap::*,
+    wrapped_command_output::*,
+    wrapped_report::*,
 };
 
 #[macro_use] extern crate cli_log;
