@@ -1,6 +1,6 @@
 use {
     crate::*,
-    anyhow::*,
+    anyhow::Result,
     crossterm::{
         cursor, execute,
         style::{Attribute, Color::*, Print},
@@ -13,17 +13,16 @@ use {
 };
 
 /// contains the currently rendered state of the application,
-/// including scroll level and the current report (if any)
-pub struct AppState {
-    /// project name: usually the name of the package directory
-    /// but might change
-    pub project_name: String,
-    /// "check", "clippy", "test", "check_windows", etc.
-    pub job_name: String,
+/// including scroll position and the current report (if any)
+pub struct AppState<'s> {
+    /// the mission to run, with settings
+    pub mission: Mission<'s>,
     /// the lines of a computation in progress
-    lines: Option<Vec<CommandOutputLine>>,
+    output: Option<CommandOutput>,
+    /// wrapped output for the width of the console
+    wrapped_output: Option<WrappedCommandOutput>,
     /// result of a command, hopefully a report
-    cmd_result: CommandResult,
+    pub cmd_result: CommandResult,
     /// a report wrapped for the size of the console
     wrapped_report: Option<WrappedReport>,
     /// screen width
@@ -38,6 +37,7 @@ pub struct AppState {
     pub backtrace: bool,
     /// whether we should display only titles and locations
     summary: bool,
+    /// whether we display the gui bottom-to-top
     reverse: bool,
     /// colors and styles used for status bar
     status_skin: MadSkin,
@@ -45,20 +45,27 @@ pub struct AppState {
     scroll: usize,
     /// item_idx of the item which was on top on last draw
     top_item_idx: usize,
+    /// the tool building the help line
+    help_line: HelpLine,
+    /// the help page displayed over the rest, if any
+    help_page: Option<HelpPage>,
+    /// display the raw output instead of the report
+    raw_output: bool,
 }
 
-impl AppState {
-    pub fn new(mission: &Mission) -> Result<Self> {
+impl<'s> AppState<'s> {
+
+    pub fn new(mission: Mission<'s>) -> Result<Self> {
         let mut status_skin = MadSkin::default();
         status_skin
             .paragraph
             .set_fgbg(AnsiValue(252), AnsiValue(239));
         status_skin.italic = CompoundStyle::new(Some(AnsiValue(204)), None, Attribute::Bold.into());
         let (width, height) = termimad::terminal_size();
+        let help_line = HelpLine::new(mission.settings);
         Ok(Self {
-            project_name: mission.package_name.clone(),
-            job_name: mission.job_name.clone(),
-            lines: None,
+            output: None,
+            wrapped_output: None,
             cmd_result: CommandResult::None,
             wrapped_report: None,
             width,
@@ -71,17 +78,30 @@ impl AppState {
             status_skin,
             scroll: 0,
             top_item_idx: 0,
+            help_line,
+            help_page: None,
+            mission,
+            raw_output: false,
         })
     }
-}
 
-impl AppState {
     pub fn add_line(&mut self, line: CommandOutputLine) {
         let auto_scroll = self.is_scroll_at_bottom();
-        self.lines.get_or_insert_with(Vec::new).push(line);
-        if auto_scroll {
-            // if the user never scrolled, we'll stick to the bottom
-            self.scroll_to_bottom();
+        if let Some(output) = self.output.as_mut() {
+            output.push(line);
+            if auto_scroll {
+                // if the user never scrolled, we'll stick to the bottom
+                self.scroll_to_bottom();
+            }
+        } else {
+            self.wrapped_output = None;
+            self.output = {
+                let mut output = CommandOutput::default();
+                output.push(line);
+                Some(output)
+            };
+            self.scroll = 0;
+            self.fix_scroll();
         }
     }
     pub fn new_task(&self) -> Task {
@@ -89,12 +109,16 @@ impl AppState {
             backtrace: self.backtrace,
         }
     }
-    pub fn take_lines(&mut self) -> Option<Vec<CommandOutputLine>> {
-        self.lines.take()
+    pub fn take_output(&mut self) -> Option<CommandOutput> {
+        self.wrapped_output = None;
+        self.output.take()
     }
     pub fn has_report(&self) -> bool {
         matches!(self.cmd_result, CommandResult::Report(_))
     }
+    pub fn toggle_raw_output(&mut self) {
+        self.raw_output ^= true;
+    }
     pub fn set_result(&mut self, mut cmd_result: CommandResult) {
         if self.reverse {
             cmd_result.reverse();
@@ -121,14 +145,23 @@ impl AppState {
                 report.lines.pop();
             }
         }
+        // we update the location file if required
+        if self.mission.settings.export_locations {
+            if let Err(err) = cmd_result.update_location_file(&self.mission) {
+                // FIXME user info
+                error!("Error writing location file: {}", err);
+            }
+        }
         // we keep the scroll when the number of lines didn't change
         let reset_scroll = self.cmd_result.lines_len() != cmd_result.lines_len();
         self.wrapped_report = None;
+        self.wrapped_output = None;
         self.cmd_result = cmd_result;
         self.computing = false;
         if reset_scroll {
             self.reset_scroll();
         }
+        self.raw_output = false;
     }
     pub fn computation_starts(&mut self) {
         self.computing = true;
@@ -190,10 +223,32 @@ impl AppState {
         }
         0
     }
+    pub fn keybindings(&self) -> &KeyBindings {
+        &self.mission.settings.keybindings
+    }
     fn try_scroll_to_last_top_item(&mut self) {
         self.scroll = self.get_last_item_scroll();
         self.fix_scroll();
     }
+    /// close the help and return true if it was open,
+    /// return false otherwise
+    pub fn close_help(&mut self) -> bool {
+        if self.help_page.is_some() {
+            self.help_page = None;
+            true
+        } else {
+            false
+        }
+    }
+    pub fn is_help(&self) -> bool {
+        self.help_page.is_some()
+    }
+    pub fn toggle_help(&mut self) {
+        self.help_page = match self.help_page {
+            Some(_) => None,
+            None => Some(HelpPage::new(self.mission.settings)),
+        };
+    }
     pub fn toggle_summary_mode(&mut self) {
         self.summary ^= true;
         self.try_scroll_to_last_top_item();
@@ -209,9 +264,13 @@ impl AppState {
     }
     fn content_height(&self) -> usize {
         if let CommandResult::Report(report) = &self.cmd_result {
-            report.stats.lines(self.summary)
-        } else if let Some(lines) = &self.lines {
-            lines.len()
+            if report.is_success() || self.raw_output {
+                report.output.len()
+            } else {
+                report.stats.lines(self.summary)
+            }
+        } else if let Some(output) = &self.output {
+            output.len()
         } else {
             0
         }
@@ -222,38 +281,27 @@ impl AppState {
     pub fn resize(&mut self, width: u16, height: u16) {
         if self.width != width {
             self.wrapped_report = None;
+            self.wrapped_output = None;
         }
         self.width = width;
         self.height = height;
         self.try_scroll_to_last_top_item();
     }
-    fn apply_scroll_command(&mut self, cmd: ScrollCommand) {
-        self.scroll = cmd.apply(self.scroll, self.content_height(), self.page_height());
-    }
-    pub fn scroll(&mut self, w: &mut W, cmd: ScrollCommand) -> Result<()> {
-        self.apply_scroll_command(cmd);
-        self.draw(w)
-    }
-    fn draw_status(&self, w: &mut W, y: u16) -> Result<()> {
-        let mut parts = vec!["hit *q* to quit"];
-        if let CommandResult::Report(report) = &self.cmd_result {
-            if report.suggest_backtrace {
-                parts.push("*t* to toggle backtraces");
-            } else {
-                parts.push("*s* to toggle summary mode");
-                if self.wrap {
-                    parts.push("*w* to not wrap lines");
-                } else {
-                    parts.push("*w* to wrap lines");
-                }
-            }
+    pub fn apply_scroll_command(&mut self, cmd: ScrollCommand) {
+        if let Some(help_page) = self.help_page.as_mut() {
+            help_page.apply_scroll_command(cmd);
+        } else {
+            self.scroll = cmd.apply(self.scroll, self.content_height(), self.page_height());
         }
-        let status = parts.join(", ");
+    }
+    /// draw the grey line containing the keybindings indications
+    fn draw_help_line(&self, w: &mut W, y: u16) -> Result<()> {
+        let markdown = self.help_line.markdown(self);
         if self.height > 1 {
             goto(w, y)?;
             self.status_skin.write_composite_fill(
                 w,
-                Composite::from_inline(&status),
+                Composite::from_inline(&markdown),
                 self.width.into(),
                 Alignment::Left,
             )?;
@@ -265,9 +313,10 @@ impl AppState {
         goto(w, y)?;
         let mut t_line = TLine::default();
         // white over grey
-        t_line.add_badge(TString::badge(&self.project_name, 255, 240));
+        let project_name = &self.mission.location_name;
+        t_line.add_badge(TString::badge(project_name, 255, 240));
         // black over pink
-        t_line.add_badge(TString::badge(&self.job_name, 235, 204));
+        t_line.add_badge(TString::badge(&self.mission.job_name, 235, 204));
         if let CommandResult::Report(report) = &self.cmd_result {
             let stats = &report.stats;
             if stats.errors > 0 {
@@ -310,6 +359,13 @@ impl AppState {
         }
         Ok(())
     }
+    /// the action to execute now
+    pub fn action(&self) -> Option<&Action> {
+        self.mission
+            .on_success()
+            .as_ref()
+            .filter(|_| self.cmd_result.is_success())
+    }
     /// draw the report or the lines of the current computation, between
     /// y and self.page_height()
     pub fn draw_content(&mut self, w: &mut W, y: u16) -> Result<()> {
@@ -334,15 +390,20 @@ impl AppState {
             goto(w, y)?;
             clear_line(w)?;
         }
-        if let CommandResult::Report(report) = &self.cmd_result {
+        let report_to_draw = self.cmd_result
+            .report()
+            .filter(|_| !self.raw_output)
+            .filter(|report| !report.is_success());
+        if let Some(report) = report_to_draw {
             if self.wrap {
-                if self.wrapped_report.is_none() {
-                    self.wrapped_report = Some(WrappedReport::new(report, self.width));
-                    self.scroll = self.get_last_item_scroll();
-                }
-                // SAFETY: we just ensured it's here
-                // (will be cleaner when Option::insert is available in stable)
-                let wrapped_report = self.wrapped_report.as_ref().unwrap();
+                let wrapped_report = match self.wrapped_report.as_mut() {
+                    None => {
+                        let wr = WrappedReport::new(report, area.width);
+                        self.scroll = self.get_last_item_scroll();
+                        self.wrapped_report.insert(wr)
+                    }
+                    Some(wr) => wr,
+                };
                 let mut sub_lines = wrapped_report
                     .sub_lines
                     .iter()
@@ -357,7 +418,7 @@ impl AppState {
                         top_item_idx.get_or_insert_with(|| sub_line.src_line(report).item_idx);
                         sub_line.draw_line_type(w, report)?;
                         write!(w, " ")?;
-                        sub_line.draw(w, report)?;
+                        sub_line.draw(w, &report.lines)?;
                     }
                     clear_line(w)?;
                     if is_thumb(y.into(), scrollbar) {
@@ -394,13 +455,39 @@ impl AppState {
                 }
             }
             self.top_item_idx = top_item_idx.unwrap_or(0);
-        } else {
-            // report hasn't yet been computed or there was an error in command
-            let lines = match &self.cmd_result {
-                CommandResult::Failure(failure) => Some(&failure.lines),
-                _ => self.lines.as_ref(),
-            };
-            if let Some(lines) = lines {
+        } else if let Some(output) = self.cmd_result.output().or(self.output.as_ref()) {
+            debug!("MODE output");
+            if self.wrap {
+                debug!("wrap mode");
+                let wrapped_output = match self.wrapped_output.as_mut() {
+                    None => {
+                        let wo = WrappedCommandOutput::new(output, area.width);
+                        self.scroll = 0;
+                        self.wrapped_output.insert(wo)
+                    }
+                    Some(wo) => {
+                        wo.update(output, area.width);
+                        wo
+                    }
+                };
+                let mut sub_lines = wrapped_output
+                    .sub_lines
+                    .iter()
+                    .skip(self.scroll);
+                for row_idx in 0..area.height {
+                    let y = row_idx + top;
+                    goto(w, y)?;
+                    if let Some(sub_line) = sub_lines.next() {
+                        sub_line.draw(w, &output.lines)?;
+                    }
+                    clear_line(w)?;
+                    if is_thumb(y.into(), scrollbar) {
+                        execute!(w, cursor::MoveTo(area.width, y), Print('▐'.to_string()))?;
+                    }
+                }
+
+            } else {
+                let lines = &output.lines;
                 for row_idx in 0..area.height {
                     let y = row_idx + top;
                     goto(w, y)?;
@@ -419,17 +506,26 @@ impl AppState {
     /// draw the state on the whole terminal
     pub fn draw(&mut self, w: &mut W) -> Result<()> {
         if self.reverse {
-            self.draw_status(w, 0)?;
-            self.draw_content(w, 1)?;
-            self.draw_computing(w, self.height - 2)?;
-            self.draw_badges(w, self.height - 1)?;
+            self.draw_help_line(w, 0)?;
+            if let Some(help_page) = self.help_page.as_mut() {
+                help_page.draw(w, Area::new(0, 1, self.width, self.height-1))?;
+            } else {
+                self.draw_content(w, 1)?;
+                self.draw_computing(w, self.height - 2)?;
+                self.draw_badges(w, self.height - 1)?;
+            }
         } else {
-            self.draw_badges(w, 0)?;
-            self.draw_computing(w, 1)?;
-            self.draw_content(w, 2)?;
-            self.draw_status(w, self.height - 1)?;
+            if let Some(help_page) = self.help_page.as_mut() {
+                help_page.draw(w, Area::new(0, 0, self.width, self.height-1))?;
+            } else {
+                self.draw_badges(w, 0)?;
+                self.draw_computing(w, 1)?;
+                self.draw_content(w, 2)?;
+            }
+            self.draw_help_line(w, self.height - 1)?;
         }
         w.flush()?;
         Ok(())
     }
 }
+
